@@ -1,3 +1,4 @@
+#[cfg(feature = "crypto")]
 use std::fmt;
 
 #[cfg(feature = "pem")]
@@ -26,51 +27,57 @@ use crate::sign_algo::{algo::*, SignAlgo};
 #[cfg(feature = "pem")]
 use crate::ENCODE_CONFIG;
 use crate::{sign_algo::SignatureAlgorithm, Error};
+#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+use aws_lc_rs::unstable::signature::PqdsaKeyPair;
 
 /// A key pair variant
 #[allow(clippy::large_enum_variant)]
+#[cfg(feature = "crypto")]
 pub(crate) enum KeyPairKind {
 	/// A Ecdsa key pair
-	#[cfg(feature = "crypto")]
 	Ec(EcdsaKeyPair),
 	/// A Ed25519 key pair
-	#[cfg(feature = "crypto")]
 	Ed(Ed25519KeyPair),
+	/// A Pqdsa key pair
+	#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+	Pq(PqdsaKeyPair),
 	/// A RSA key pair
-	#[cfg(feature = "crypto")]
 	Rsa(RsaKeyPair, &'static dyn RsaEncoding),
-	/// A remote key pair
-	Remote(Box<dyn RemoteKeyPair + Send + Sync>),
 }
 
+#[cfg(feature = "crypto")]
 impl fmt::Debug for KeyPairKind {
 	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
 		match self {
-			#[cfg(feature = "crypto")]
-			Self::Ec(key_pair) => write!(f, "{:?}", key_pair),
-			#[cfg(feature = "crypto")]
-			Self::Ed(key_pair) => write!(f, "{:?}", key_pair),
-			#[cfg(feature = "crypto")]
-			Self::Rsa(key_pair, _) => write!(f, "{:?}", key_pair),
-			Self::Remote(_) => write!(f, "Box<dyn RemotePrivateKey>"),
+			Self::Ec(key_pair) => write!(f, "{key_pair:?}"),
+			Self::Ed(key_pair) => write!(f, "{key_pair:?}"),
+			#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+			Self::Pq(key_pair) => write!(f, "{key_pair:?}"),
+			Self::Rsa(key_pair, _) => write!(f, "{key_pair:?}"),
 		}
 	}
 }
 
 /// A key pair used to sign certificates and CSRs
-///
-/// Note that ring, the underlying library to handle RSA keys
-/// requires them to be in a special format, meaning that
-/// `openssl genrsa` doesn't work. See ring's [documentation](ring::signature::RsaKeyPair::from_pkcs8)
-/// for how to generate RSA keys in the wanted format
-/// and conversion between the formats.
-#[derive(Debug)]
+#[cfg(feature = "crypto")]
 pub struct KeyPair {
 	pub(crate) kind: KeyPairKind,
 	pub(crate) alg: &'static SignatureAlgorithm,
 	pub(crate) serialized_der: Vec<u8>,
 }
 
+#[cfg(feature = "crypto")]
+impl fmt::Debug for KeyPair {
+	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+		f.debug_struct("KeyPair")
+			.field("kind", &self.kind)
+			.field("alg", &self.alg)
+			.field("serialized_der", &"[secret key elided]")
+			.finish()
+	}
+}
+
+#[cfg(feature = "crypto")]
 impl KeyPair {
 	/// Generate a new random [`PKCS_ECDSA_P256_SHA256`] key pair
 	#[cfg(feature = "crypto")]
@@ -111,6 +118,17 @@ impl KeyPair {
 					serialized_der: key_pair_serialized,
 				})
 			},
+			#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+			SignAlgo::PqDsa(sign_alg) => {
+				let key_pair = PqdsaKeyPair::generate(sign_alg)._err()?;
+				let key_pair_serialized = key_pair.to_pkcs8()._err()?.as_ref().to_vec();
+
+				Ok(KeyPair {
+					kind: KeyPairKind::Pq(key_pair),
+					alg,
+					serialized_der: key_pair_serialized,
+				})
+			},
 			#[cfg(feature = "aws_lc_rs")]
 			SignAlgo::Rsa(sign_alg) => Self::generate_rsa_inner(alg, sign_alg, KeySize::Rsa2048),
 			// Ring doesn't have RSA key generation yet:
@@ -178,15 +196,6 @@ impl KeyPair {
 		Self::try_from(private_key.contents())
 	}
 
-	/// Obtains the key pair from a raw public key and a remote private key
-	pub fn from_remote(key_pair: Box<dyn RemoteKeyPair + Send + Sync>) -> Result<Self, Error> {
-		Ok(Self {
-			alg: key_pair.algorithm(),
-			kind: KeyPairKind::Remote(key_pair),
-			serialized_der: Vec::new(),
-		})
-	}
-
 	/// Obtains the key pair from a DER formatted key
 	/// using the specified [`SignatureAlgorithm`]
 	///
@@ -381,9 +390,12 @@ impl KeyPair {
 
 	/// Get the raw public key of this key pair
 	///
-	/// The key is in raw format, as how [`ring::signature::KeyPair::public_key`]
-	/// would output, and how [`ring::signature::UnparsedPublicKey::verify`]
+	/// The key is in raw format, as how [`KeyPair::public_key()`][public_key]
+	/// would output, and how [`UnparsedPublicKey::verify()`][verify]
 	/// would accept.
+	///
+	/// [public_key]: crate::ring_like::signature::KeyPair::public_key()
+	/// [verify]: crate::ring_like::signature::UnparsedPublicKey::verify()
 	pub fn public_key_raw(&self) -> &[u8] {
 		self.der_bytes()
 	}
@@ -399,112 +411,27 @@ impl KeyPair {
 		std::iter::once(self.alg)
 	}
 
-	pub(crate) fn sign_der(
-		&self,
-		f: impl FnOnce(&mut DERWriterSeq<'_>) -> Result<(), Error>,
-	) -> Result<Vec<u8>, Error> {
-		yasna::try_construct_der(|writer| {
-			writer.write_sequence(|writer| {
-				let data = yasna::try_construct_der(|writer| writer.write_sequence(f))?;
-				writer.next().write_der(&data);
-
-				// Write signatureAlgorithm
-				self.alg.write_alg_ident(writer.next());
-
-				// Write signature
-				self.sign(&data, writer.next())?;
-
-				Ok(())
-			})
-		})
-	}
-
-	pub(crate) fn sign(&self, msg: &[u8], writer: DERWriter) -> Result<(), Error> {
-		match &self.kind {
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Ec(kp) => {
-				let system_random = SystemRandom::new();
-				let signature = kp.sign(&system_random, msg)._err()?;
-				let sig = &signature.as_ref();
-				writer.write_bitvec_bytes(sig, &sig.len() * 8);
-			},
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Ed(kp) => {
-				let signature = kp.sign(msg);
-				let sig = &signature.as_ref();
-				writer.write_bitvec_bytes(sig, &sig.len() * 8);
-			},
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Rsa(kp, padding_alg) => {
-				let system_random = SystemRandom::new();
-				let mut signature = vec![0; rsa_key_pair_public_modulus_len(kp)];
-				kp.sign(*padding_alg, &system_random, msg, &mut signature)
-					._err()?;
-				let sig = &signature.as_ref();
-				writer.write_bitvec_bytes(sig, &sig.len() * 8);
-			},
-			KeyPairKind::Remote(kp) => {
-				let signature = kp.sign(msg)?;
-				writer.write_bitvec_bytes(&signature, &signature.len() * 8);
-			},
-		}
-		Ok(())
-	}
-
-	/// Return the key pair's public key in DER format
-	///
-	/// The key is formatted according to the SubjectPublicKeyInfo struct of
-	/// X.509.
-	/// See [RFC 5280 section 4.1](https://tools.ietf.org/html/rfc5280#section-4.1).
-	pub fn public_key_der(&self) -> Vec<u8> {
-		yasna::construct_der(|writer| serialize_public_key_der(self, writer))
-	}
-
 	/// Return the key pair's public key in PEM format
 	///
 	/// The returned string can be interpreted with `openssl pkey --inform PEM -pubout -pubin -text`
 	#[cfg(feature = "pem")]
 	pub fn public_key_pem(&self) -> String {
-		let contents = self.public_key_der();
+		let contents = self.subject_public_key_info();
 		let p = Pem::new("PUBLIC KEY", contents);
 		pem::encode_config(&p, ENCODE_CONFIG)
 	}
 
 	/// Serializes the key pair (including the private key) in PKCS#8 format in DER
-	///
-	/// Panics if called on a remote key pair.
 	pub fn serialize_der(&self) -> Vec<u8> {
-		#[cfg_attr(not(feature = "crypto"), allow(irrefutable_let_patterns))]
-		if let KeyPairKind::Remote(_) = self.kind {
-			panic!("Serializing a remote key pair is not supported")
-		}
-
 		self.serialized_der.clone()
 	}
 
 	/// Returns a reference to the serialized key pair (including the private key)
 	/// in PKCS#8 format in DER
-	///
-	/// Panics if called on a remote key pair.
 	pub fn serialized_der(&self) -> &[u8] {
-		#[cfg_attr(not(feature = "crypto"), allow(irrefutable_let_patterns))]
-		if let KeyPairKind::Remote(_) = self.kind {
-			panic!("Serializing a remote key pair is not supported")
-		}
-
 		&self.serialized_der
 	}
 
-	/// Access the remote key pair if it is a remote one
-	pub fn as_remote(&self) -> Option<&(dyn RemoteKeyPair + Send + Sync)> {
-		#[cfg_attr(not(feature = "crypto"), allow(irrefutable_let_patterns))]
-		if let KeyPairKind::Remote(remote) = &self.kind {
-			Some(remote.as_ref())
-		} else {
-			None
-		}
-	}
-
 	/// Serializes the key pair (including the private key) in PKCS#8 format in PEM
 	#[cfg(feature = "pem")]
 	pub fn serialize_pem(&self) -> String {
@@ -514,6 +441,50 @@ impl KeyPair {
 	}
 }
 
+#[cfg(feature = "crypto")]
+impl SigningKey for KeyPair {
+	fn sign(&self, msg: &[u8]) -> Result<Vec<u8>, Error> {
+		Ok(match &self.kind {
+			KeyPairKind::Ec(kp) => {
+				let system_random = SystemRandom::new();
+				let signature = kp.sign(&system_random, msg)._err()?;
+				signature.as_ref().to_owned()
+			},
+			KeyPairKind::Ed(kp) => kp.sign(msg).as_ref().to_owned(),
+			#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+			KeyPairKind::Pq(kp) => {
+				let mut signature = vec![0; kp.algorithm().signature_len()];
+				kp.sign(msg, &mut signature)._err()?;
+				signature
+			},
+			KeyPairKind::Rsa(kp, padding_alg) => {
+				let system_random = SystemRandom::new();
+				let mut signature = vec![0; rsa_key_pair_public_modulus_len(kp)];
+				kp.sign(*padding_alg, &system_random, msg, &mut signature)
+					._err()?;
+				signature
+			},
+		})
+	}
+}
+
+#[cfg(feature = "crypto")]
+impl PublicKeyData for KeyPair {
+	fn der_bytes(&self) -> &[u8] {
+		match &self.kind {
+			KeyPairKind::Ec(kp) => kp.public_key().as_ref(),
+			KeyPairKind::Ed(kp) => kp.public_key().as_ref(),
+			#[cfg(all(feature = "aws_lc_rs_unstable", not(feature = "fips")))]
+			KeyPairKind::Pq(kp) => kp.public_key().as_ref(),
+			KeyPairKind::Rsa(kp, _) => kp.public_key().as_ref(),
+		}
+	}
+
+	fn algorithm(&self) -> &'static SignatureAlgorithm {
+		self.alg
+	}
+}
+
 #[cfg(feature = "crypto")]
 impl TryFrom<&[u8]> for KeyPair {
 	type Error = Error;
@@ -636,37 +607,32 @@ pub enum RsaKeySize {
 	_4096,
 }
 
-impl PublicKeyData for KeyPair {
-	fn der_bytes(&self) -> &[u8] {
-		match &self.kind {
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Ec(kp) => kp.public_key().as_ref(),
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Ed(kp) => kp.public_key().as_ref(),
-			#[cfg(feature = "crypto")]
-			KeyPairKind::Rsa(kp, _) => kp.public_key().as_ref(),
-			KeyPairKind::Remote(kp) => kp.public_key(),
-		}
-	}
+pub(crate) fn sign_der(
+	key: &impl SigningKey,
+	f: impl FnOnce(&mut DERWriterSeq<'_>) -> Result<(), Error>,
+) -> Result<Vec<u8>, Error> {
+	yasna::try_construct_der(|writer| {
+		writer.write_sequence(|writer| {
+			let data = yasna::try_construct_der(|writer| writer.write_sequence(f))?;
+			writer.next().write_der(&data);
 
-	fn algorithm(&self) -> &SignatureAlgorithm {
-		self.alg
-	}
+			// Write signatureAlgorithm
+			key.algorithm().write_alg_ident(writer.next());
+
+			// Write signature
+			let sig = key.sign(&data)?;
+			let writer = writer.next();
+			writer.write_bitvec_bytes(&sig, sig.len() * 8);
+
+			Ok(())
+		})
+	})
 }
 
-/// A private key that is not directly accessible, but can be used to sign messages
-///
-/// Trait objects based on this trait can be passed to the [`KeyPair::from_remote`] function for generating certificates
-/// from a remote and raw private key, for example an HSM.
-pub trait RemoteKeyPair {
-	/// Returns the public key of this key pair in the binary format as in [`KeyPair::public_key_raw`]
-	fn public_key(&self) -> &[u8];
-
+/// A key that can be used to sign messages
+pub trait SigningKey: PublicKeyData {
 	/// Signs `msg` using the selected algorithm
 	fn sign(&self, msg: &[u8]) -> Result<Vec<u8>, Error>;
-
-	/// Reveals the algorithm to be used when calling `sign()`
-	fn algorithm(&self) -> &'static SignatureAlgorithm;
 }
 
 #[cfg(feature = "crypto")]
@@ -747,21 +713,29 @@ impl PublicKeyData for SubjectPublicKeyInfo {
 		&self.subject_public_key
 	}
 
-	fn algorithm(&self) -> &SignatureAlgorithm {
+	fn algorithm(&self) -> &'static SignatureAlgorithm {
 		self.alg
 	}
 }
 
 /// The public key data of a key pair
 pub trait PublicKeyData {
+	/// The public key data in DER format
+	///
+	/// The key is formatted according to the X.509 SubjectPublicKeyInfo struct.
+	/// See [RFC 5280 section 4.1](https://tools.ietf.org/html/rfc5280#section-4.1).
+	fn subject_public_key_info(&self) -> Vec<u8> {
+		yasna::construct_der(|writer| serialize_public_key_der(self, writer))
+	}
+
 	/// The public key in DER format
 	fn der_bytes(&self) -> &[u8];
 
 	/// The algorithm used by the key pair
-	fn algorithm(&self) -> &SignatureAlgorithm;
+	fn algorithm(&self) -> &'static SignatureAlgorithm;
 }
 
-pub(crate) fn serialize_public_key_der(key: &impl PublicKeyData, writer: DERWriter) {
+pub(crate) fn serialize_public_key_der(key: &(impl PublicKeyData + ?Sized), writer: DERWriter) {
 	writer.write_sequence(|writer| {
 		key.algorithm().write_oids_sign_alg(writer.next());
 		let pk = key.der_bytes();
@@ -792,7 +766,7 @@ mod test {
 		] {
 			let kp = KeyPair::generate_for(alg).expect("keygen");
 			let pem = kp.public_key_pem();
-			let der = kp.public_key_der();
+			let der = kp.subject_public_key_info();
 
 			let pkd_pem = SubjectPublicKeyInfo::from_pem(&pem).expect("from pem");
 			assert_eq!(kp.der_bytes(), pkd_pem.der_bytes());
